package info.bayro.baFwk.valueObject;

import info.bayro.baFwk.exception.InvalidValueException;
import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.util.HashSet;
import java.util.Set;
import org.apache.log4j.Logger;


public class ValueChecker {

	public static final Logger log = Logger.getLogger(ValueChecker.class);


	public static ValueDescriptor getValueDescriptor(ValueObject valueObject, String fieldName)
			throws SecurityException, NoSuchFieldException {

		if (log.isTraceEnabled()) {
			log.trace("Param: " + valueObject.getClass() + ", " + fieldName);
		}

		Field field = valueObject.getField(fieldName);

		ValueDescriptor descriptor = null;

		// check the field for annotations
		Annotation[] annotations = field.getAnnotations();
		for (Annotation annotation : annotations) {
			if (annotation instanceof ValueDescriptor) {
				descriptor = (ValueDescriptor) annotation;
			}
		}

		// if the field is a own datatype
		if (descriptor == null) {

			annotations = field.getType().getAnnotations();
			for (Annotation annotation : annotations) {
				if (annotation instanceof ValueDescriptor) {
					descriptor = (ValueDescriptor) annotation;
				}
			}

		}

		return descriptor;
	}


	public static boolean checkValue(ValueObject valueObject, String fieldName, Object value)
			throws InvalidValueException {

		if (log.isTraceEnabled()) {
			log.trace("Param: " + valueObject.getClass() + ", " + fieldName + ", " + value);
		}

		ValueDescriptor valueDescriptor = null;
		try {
			valueDescriptor = ValueChecker.getValueDescriptor(valueObject, fieldName);
		} catch (SecurityException e) {
			throw new InvalidValueException(
					ValueGlobals._MSG_FIELD_UNDEFINED_VALUEDESCRIPTOR,
					valueObject.getClass(),
					fieldName,
					value);

		} catch (NoSuchFieldException e) {
			throw new InvalidValueException(
					ValueGlobals._MSG_FIELD_UNDEFINED_VALUEDESCRIPTOR,
					valueObject.getClass(),
					fieldName,
					value);
		}

		if (value == null) {
			return true;
		}

		ValueFieldType fieldType = valueDescriptor.fieldType();
		int maxSize = valueDescriptor.maxSize();
		int fieldDigits = valueDescriptor.fieldDigits();

		String str = value.toString().trim();
		if (str.isEmpty() || str.length() == 0) {
			return true;
		}

		Set<Character> symbol = createSet(fieldType, fieldDigits);

		if (fieldType == ValueFieldType._FIELD_ALPHANUM) {

			// check symbol
			for (Character character : str.toCharArray()) {
				if (!symbol.contains(character)) {
					throw new InvalidValueException(
							ValueGlobals._MSG_FIELD_DISALLOWED_SYMBOL,
							valueObject.getClass(),
							fieldName,
							value);
				}
			}

			if (str.length() <= maxSize) {
				return true;

			} else {
				throw new InvalidValueException(
						ValueGlobals._MSG_FIELD_TO_LONG,
						valueObject.getClass(),
						fieldName);
			}

		}

		if (fieldType == ValueFieldType._FIELD_SIGNED ||
				fieldType == ValueFieldType._FIELD_UNSIGNED) {

			str = str.replace(".", ",");

			// check symbol
			for (Character character : str.toCharArray()) {

				if (!symbol.contains(character)) {
					throw new InvalidValueException(
							ValueGlobals._MSG_FIELD_DISALLOWED_SYMBOL,
							valueObject.getClass(),
							fieldName,
							value);
				}
			}

			// check prefix
			if (fieldType == ValueFieldType._FIELD_SIGNED && str.contains("-")) {
				str = str.replace("-", "");
			}

			String[] split = str.split(",");

			if (split[0] != null && split[0].length() > maxSize) {
				throw new InvalidValueException(
						ValueGlobals._MSG_FIELD_TO_LONG,
						valueObject.getClass(),
						fieldName,
						value);
			}

			if (split.length >= 2 && split[1] != null && split[1].length() > fieldDigits) {
				throw new InvalidValueException(
						ValueGlobals._MSG_FIELD_TO_LONG,
						valueObject.getClass(),
						fieldName,
						value);
			}

			return true;
		}

		throw new InvalidValueException(
				ValueGlobals._MSG_FIELD_UNDEFINED_TYPE,
				valueObject.getClass(),
				fieldName);

	}


	public static Set<Character> createSet(ValueFieldType fieldType, int fieldDigits) {
		if (log.isTraceEnabled()) {
			log.trace("Param: " + fieldType + ", " + fieldDigits);
		}

		Set<Character> tmp = new HashSet<Character>();

		char[] charArray = ValueGlobals._SYMBOL_NUM;
		for (int i = 0; i < charArray.length; i++) {
			tmp.add(charArray[i]);
		}

		if (fieldType == ValueFieldType._FIELD_ALPHANUM || fieldDigits > 0) {
			charArray = ValueGlobals._SYMBOL_NUM_DEC_POINT;
			for (int i = 0; i < charArray.length; i++) {
				tmp.add(charArray[i]);
			}
		}

		if (fieldType == ValueFieldType._FIELD_ALPHANUM || fieldType == ValueFieldType._FIELD_SIGNED) {
			charArray = ValueGlobals._SYMBOL_NUM_SIGN_NEG;
			for (int i = 0; i < charArray.length; i++) {
				tmp.add(charArray[i]);
			}
		}

		if (fieldType == ValueFieldType._FIELD_ALPHANUM) {
			// default alphanumerisch
			charArray = ValueGlobals._SYMBOL_ALPHA_CAPITAL_LETTER;
			for (int i = 0; i < charArray.length; i++) {
				tmp.add(charArray[i]);
			}

			charArray = ValueGlobals._SYMBOL_ALPHA_LOWER_CASE;
			for (int i = 0; i < charArray.length; i++) {
				tmp.add(charArray[i]);
			}

			charArray = ValueGlobals._SYMBOL_ADD_CHAR;
			for (int i = 0; i < charArray.length; i++) {
				tmp.add(charArray[i]);
			}

			charArray = ValueGlobals._SYMBOL_NUM_SIGN_POS;
			for (int i = 0; i < charArray.length; i++) {
				tmp.add(charArray[i]);
			}
		}

		return tmp;
	}

}